# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast
from typing_extensions import Literal

import httpx

from ......_types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ......_utils import path_template, maybe_transform, async_maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_wrappers import ResultWrapper
from ......pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ......_base_client import AsyncPaginator, make_request_options
from ......types.zero_trust.access.ai_controls.mcp import server_list_params, server_create_params, server_update_params
from ......types.zero_trust.access.ai_controls.mcp.server_list_response import ServerListResponse
from ......types.zero_trust.access.ai_controls.mcp.server_read_response import ServerReadResponse
from ......types.zero_trust.access.ai_controls.mcp.server_create_response import ServerCreateResponse
from ......types.zero_trust.access.ai_controls.mcp.server_delete_response import ServerDeleteResponse
from ......types.zero_trust.access.ai_controls.mcp.server_update_response import ServerUpdateResponse

__all__ = ["ServersResource", "AsyncServersResource"]


class ServersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ServersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ServersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ServersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ServersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        id: str,
        auth_type: Literal["oauth", "bearer", "unauthenticated"],
        hostname: str,
        name: str,
        auth_credentials: str | Omit = omit,
        description: Optional[str] | Omit = omit,
        updated_prompts: Iterable[server_create_params.UpdatedPrompt] | Omit = omit,
        updated_tools: Iterable[server_create_params.UpdatedTool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServerCreateResponse:
        """
        Creates a new MCP portal for managing AI tool access through Cloudflare Access.

        Args:
          id: server id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/servers", account_id=account_id),
            body=maybe_transform(
                {
                    "id": id,
                    "auth_type": auth_type,
                    "hostname": hostname,
                    "name": name,
                    "auth_credentials": auth_credentials,
                    "description": description,
                    "updated_prompts": updated_prompts,
                    "updated_tools": updated_tools,
                },
                server_create_params.ServerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ServerCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[ServerCreateResponse], ResultWrapper[ServerCreateResponse]),
        )

    def update(
        self,
        id: str,
        *,
        account_id: str,
        auth_credentials: str | Omit = omit,
        description: Optional[str] | Omit = omit,
        name: str | Omit = omit,
        updated_prompts: Iterable[server_update_params.UpdatedPrompt] | Omit = omit,
        updated_tools: Iterable[server_update_params.UpdatedTool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServerUpdateResponse:
        """
        Updates an MCP portal configuration.

        Args:
          id: server id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/servers/{id}", account_id=account_id, id=id),
            body=maybe_transform(
                {
                    "auth_credentials": auth_credentials,
                    "description": description,
                    "name": name,
                    "updated_prompts": updated_prompts,
                    "updated_tools": updated_tools,
                },
                server_update_params.ServerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ServerUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[ServerUpdateResponse], ResultWrapper[ServerUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[ServerListResponse]:
        """
        Lists all MCP portals configured for the account.

        Args:
          search: Search by id, name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/servers", account_id=account_id),
            page=SyncV4PagePaginationArray[ServerListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    server_list_params.ServerListParams,
                ),
            ),
            model=ServerListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServerDeleteResponse:
        """
        Deletes an MCP portal from the account.

        Args:
          id: server id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/servers/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ServerDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[ServerDeleteResponse], ResultWrapper[ServerDeleteResponse]),
        )

    def read(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServerReadResponse:
        """
        Retrieves gateway configuration for MCP portals.

        Args:
          id: server id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/servers/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ServerReadResponse]._unwrapper,
            ),
            cast_to=cast(Type[ServerReadResponse], ResultWrapper[ServerReadResponse]),
        )

    def sync(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Syncs an MCP server's tool catalog with the portal.

        Args:
          id: portal id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/access/ai-controls/mcp/servers/{id}/sync", account_id=account_id, id=id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class AsyncServersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncServersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncServersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncServersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncServersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        id: str,
        auth_type: Literal["oauth", "bearer", "unauthenticated"],
        hostname: str,
        name: str,
        auth_credentials: str | Omit = omit,
        description: Optional[str] | Omit = omit,
        updated_prompts: Iterable[server_create_params.UpdatedPrompt] | Omit = omit,
        updated_tools: Iterable[server_create_params.UpdatedTool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServerCreateResponse:
        """
        Creates a new MCP portal for managing AI tool access through Cloudflare Access.

        Args:
          id: server id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/servers", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "id": id,
                    "auth_type": auth_type,
                    "hostname": hostname,
                    "name": name,
                    "auth_credentials": auth_credentials,
                    "description": description,
                    "updated_prompts": updated_prompts,
                    "updated_tools": updated_tools,
                },
                server_create_params.ServerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ServerCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[ServerCreateResponse], ResultWrapper[ServerCreateResponse]),
        )

    async def update(
        self,
        id: str,
        *,
        account_id: str,
        auth_credentials: str | Omit = omit,
        description: Optional[str] | Omit = omit,
        name: str | Omit = omit,
        updated_prompts: Iterable[server_update_params.UpdatedPrompt] | Omit = omit,
        updated_tools: Iterable[server_update_params.UpdatedTool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServerUpdateResponse:
        """
        Updates an MCP portal configuration.

        Args:
          id: server id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/servers/{id}", account_id=account_id, id=id),
            body=await async_maybe_transform(
                {
                    "auth_credentials": auth_credentials,
                    "description": description,
                    "name": name,
                    "updated_prompts": updated_prompts,
                    "updated_tools": updated_tools,
                },
                server_update_params.ServerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ServerUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[ServerUpdateResponse], ResultWrapper[ServerUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ServerListResponse, AsyncV4PagePaginationArray[ServerListResponse]]:
        """
        Lists all MCP portals configured for the account.

        Args:
          search: Search by id, name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/servers", account_id=account_id),
            page=AsyncV4PagePaginationArray[ServerListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    server_list_params.ServerListParams,
                ),
            ),
            model=ServerListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServerDeleteResponse:
        """
        Deletes an MCP portal from the account.

        Args:
          id: server id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/servers/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ServerDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[ServerDeleteResponse], ResultWrapper[ServerDeleteResponse]),
        )

    async def read(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServerReadResponse:
        """
        Retrieves gateway configuration for MCP portals.

        Args:
          id: server id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/servers/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ServerReadResponse]._unwrapper,
            ),
            cast_to=cast(Type[ServerReadResponse], ResultWrapper[ServerReadResponse]),
        )

    async def sync(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Syncs an MCP server's tool catalog with the portal.

        Args:
          id: portal id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/access/ai-controls/mcp/servers/{id}/sync", account_id=account_id, id=id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class ServersResourceWithRawResponse:
    def __init__(self, servers: ServersResource) -> None:
        self._servers = servers

        self.create = to_raw_response_wrapper(
            servers.create,
        )
        self.update = to_raw_response_wrapper(
            servers.update,
        )
        self.list = to_raw_response_wrapper(
            servers.list,
        )
        self.delete = to_raw_response_wrapper(
            servers.delete,
        )
        self.read = to_raw_response_wrapper(
            servers.read,
        )
        self.sync = to_raw_response_wrapper(
            servers.sync,
        )


class AsyncServersResourceWithRawResponse:
    def __init__(self, servers: AsyncServersResource) -> None:
        self._servers = servers

        self.create = async_to_raw_response_wrapper(
            servers.create,
        )
        self.update = async_to_raw_response_wrapper(
            servers.update,
        )
        self.list = async_to_raw_response_wrapper(
            servers.list,
        )
        self.delete = async_to_raw_response_wrapper(
            servers.delete,
        )
        self.read = async_to_raw_response_wrapper(
            servers.read,
        )
        self.sync = async_to_raw_response_wrapper(
            servers.sync,
        )


class ServersResourceWithStreamingResponse:
    def __init__(self, servers: ServersResource) -> None:
        self._servers = servers

        self.create = to_streamed_response_wrapper(
            servers.create,
        )
        self.update = to_streamed_response_wrapper(
            servers.update,
        )
        self.list = to_streamed_response_wrapper(
            servers.list,
        )
        self.delete = to_streamed_response_wrapper(
            servers.delete,
        )
        self.read = to_streamed_response_wrapper(
            servers.read,
        )
        self.sync = to_streamed_response_wrapper(
            servers.sync,
        )


class AsyncServersResourceWithStreamingResponse:
    def __init__(self, servers: AsyncServersResource) -> None:
        self._servers = servers

        self.create = async_to_streamed_response_wrapper(
            servers.create,
        )
        self.update = async_to_streamed_response_wrapper(
            servers.update,
        )
        self.list = async_to_streamed_response_wrapper(
            servers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            servers.delete,
        )
        self.read = async_to_streamed_response_wrapper(
            servers.read,
        )
        self.sync = async_to_streamed_response_wrapper(
            servers.sync,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .updates import (
    UpdatesResource,
    AsyncUpdatesResource,
    UpdatesResourceWithRawResponse,
    AsyncUpdatesResourceWithRawResponse,
    UpdatesResourceWithStreamingResponse,
    AsyncUpdatesResourceWithStreamingResponse,
)
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource

__all__ = ["SCIMResource", "AsyncSCIMResource"]


class SCIMResource(SyncAPIResource):
    @cached_property
    def updates(self) -> UpdatesResource:
        return UpdatesResource(self._client)

    @cached_property
    def with_raw_response(self) -> SCIMResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SCIMResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SCIMResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SCIMResourceWithStreamingResponse(self)


class AsyncSCIMResource(AsyncAPIResource):
    @cached_property
    def updates(self) -> AsyncUpdatesResource:
        return AsyncUpdatesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSCIMResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSCIMResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSCIMResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSCIMResourceWithStreamingResponse(self)


class SCIMResourceWithRawResponse:
    def __init__(self, scim: SCIMResource) -> None:
        self._scim = scim

    @cached_property
    def updates(self) -> UpdatesResourceWithRawResponse:
        return UpdatesResourceWithRawResponse(self._scim.updates)


class AsyncSCIMResourceWithRawResponse:
    def __init__(self, scim: AsyncSCIMResource) -> None:
        self._scim = scim

    @cached_property
    def updates(self) -> AsyncUpdatesResourceWithRawResponse:
        return AsyncUpdatesResourceWithRawResponse(self._scim.updates)


class SCIMResourceWithStreamingResponse:
    def __init__(self, scim: SCIMResource) -> None:
        self._scim = scim

    @cached_property
    def updates(self) -> UpdatesResourceWithStreamingResponse:
        return UpdatesResourceWithStreamingResponse(self._scim.updates)


class AsyncSCIMResourceWithStreamingResponse:
    def __init__(self, scim: AsyncSCIMResource) -> None:
        self._scim = scim

    @cached_property
    def updates(self) -> AsyncUpdatesResourceWithStreamingResponse:
        return AsyncUpdatesResourceWithStreamingResponse(self._scim.updates)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

from .catch_all_action_param import CatchAllActionParam
from .catch_all_matcher_param import CatchAllMatcherParam

__all__ = ["CatchAllUpdateParams"]


class CatchAllUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    actions: Required[Iterable[CatchAllActionParam]]
    """List actions for the catch-all routing rule."""

    matchers: Required[Iterable[CatchAllMatcherParam]]
    """List of matchers for the catch-all routing rule."""

    enabled: Literal[True, False]
    """Routing rule status."""

    name: str
    """Routing rule name."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["RecordScanTriggerResponse", "Error", "ErrorSource", "Message", "MessageSource"]


class ErrorSource(BaseModel):
    pointer: Optional[str] = None


class Error(BaseModel):
    code: int

    message: str

    documentation_url: Optional[str] = None

    source: Optional[ErrorSource] = None


class MessageSource(BaseModel):
    pointer: Optional[str] = None


class Message(BaseModel):
    code: int

    message: str

    documentation_url: Optional[str] = None

    source: Optional[MessageSource] = None


class RecordScanTriggerResponse(BaseModel):
    errors: List[Error]

    messages: List[Message]

    success: Literal[True]
    """Whether the API call was successful."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .rule_list_params import RuleListParams as RuleListParams
from .filter_list_params import FilterListParams as FilterListParams
from .rule_create_params import RuleCreateParams as RuleCreateParams
from .rule_list_response import RuleListResponse as RuleListResponse
from .filter_create_params import FilterCreateParams as FilterCreateParams
from .filter_list_response import FilterListResponse as FilterListResponse
from .rule_create_response import RuleCreateResponse as RuleCreateResponse
from .filter_create_response import FilterCreateResponse as FilterCreateResponse
from .rule_bulk_delete_response import RuleBulkDeleteResponse as RuleBulkDeleteResponse
from .filter_bulk_delete_response import FilterBulkDeleteResponse as FilterBulkDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from typing_extensions import Literal, TypeAlias

from ...._models import BaseModel

__all__ = ["SettingsMultipleRequest", "SettingsMultipleRequestItem"]


class SettingsMultipleRequestItem(BaseModel):
    """Operation ID to mitigation action mappings"""

    mitigation_action: Optional[Literal["log", "block", "none"]] = None
    """When set, this applies a mitigation action to this operation

    - `log` log request when request does not conform to schema for this operation
    - `block` deny access to the site when request does not conform to schema for
      this operation
    - `none` will skip mitigation for this operation
    - `null` indicates that no operation level mitigation is in place, see Zone
      Level Schema Validation Settings for mitigation action that will be applied
    """


SettingsMultipleRequest: TypeAlias = Dict[str, SettingsMultipleRequestItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.resource_sharing import recipient_get_params, recipient_list_params, recipient_create_params
from ...types.resource_sharing.recipient_get_response import RecipientGetResponse
from ...types.resource_sharing.recipient_list_response import RecipientListResponse
from ...types.resource_sharing.recipient_create_response import RecipientCreateResponse
from ...types.resource_sharing.recipient_delete_response import RecipientDeleteResponse

__all__ = ["RecipientsResource", "AsyncRecipientsResource"]


class RecipientsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RecipientsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RecipientsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RecipientsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RecipientsResourceWithStreamingResponse(self)

    def create(
        self,
        share_id: str,
        *,
        path_account_id: str,
        body_account_id: str | Omit = omit,
        organization_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RecipientCreateResponse]:
        """
        Adds a recipient to a resource share, granting them access to the shared
        resources.

        Args:
          path_account_id: Account identifier.

          share_id: Share identifier tag.

          body_account_id: Account identifier.

          organization_id: Organization identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not path_account_id:
            raise ValueError(f"Expected a non-empty value for `path_account_id` but received {path_account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return self._post(
            path_template(
                "/accounts/{path_account_id}/shares/{share_id}/recipients",
                path_account_id=path_account_id,
                share_id=share_id,
            ),
            body=maybe_transform(
                {
                    "body_account_id": body_account_id,
                    "organization_id": organization_id,
                },
                recipient_create_params.RecipientCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RecipientCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RecipientCreateResponse]], ResultWrapper[RecipientCreateResponse]),
        )

    def list(
        self,
        share_id: str,
        *,
        account_id: str,
        include_resources: bool | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[RecipientListResponse]:
        """
        List share recipients by share ID.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          include_resources: Include resources in the response.

          page: Page number.

          per_page: Number of objects to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/recipients", account_id=account_id, share_id=share_id
            ),
            page=SyncV4PagePaginationArray[RecipientListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_resources": include_resources,
                        "page": page,
                        "per_page": per_page,
                    },
                    recipient_list_params.RecipientListParams,
                ),
            ),
            model=RecipientListResponse,
        )

    def delete(
        self,
        recipient_id: str,
        *,
        account_id: str,
        share_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RecipientDeleteResponse]:
        """
        Deletion is not immediate, an updated share recipient object with a new status
        will be returned.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          recipient_id: Share Recipient identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        if not recipient_id:
            raise ValueError(f"Expected a non-empty value for `recipient_id` but received {recipient_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/recipients/{recipient_id}",
                account_id=account_id,
                share_id=share_id,
                recipient_id=recipient_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RecipientDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RecipientDeleteResponse]], ResultWrapper[RecipientDeleteResponse]),
        )

    def get(
        self,
        recipient_id: str,
        *,
        account_id: str,
        share_id: str,
        include_resources: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RecipientGetResponse]:
        """
        Get share recipient by ID.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          recipient_id: Share Recipient identifier tag.

          include_resources: Include resources in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        if not recipient_id:
            raise ValueError(f"Expected a non-empty value for `recipient_id` but received {recipient_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/recipients/{recipient_id}",
                account_id=account_id,
                share_id=share_id,
                recipient_id=recipient_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"include_resources": include_resources}, recipient_get_params.RecipientGetParams
                ),
                post_parser=ResultWrapper[Optional[RecipientGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RecipientGetResponse]], ResultWrapper[RecipientGetResponse]),
        )


class AsyncRecipientsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRecipientsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRecipientsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRecipientsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRecipientsResourceWithStreamingResponse(self)

    async def create(
        self,
        share_id: str,
        *,
        path_account_id: str,
        body_account_id: str | Omit = omit,
        organization_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RecipientCreateResponse]:
        """
        Adds a recipient to a resource share, granting them access to the shared
        resources.

        Args:
          path_account_id: Account identifier.

          share_id: Share identifier tag.

          body_account_id: Account identifier.

          organization_id: Organization identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not path_account_id:
            raise ValueError(f"Expected a non-empty value for `path_account_id` but received {path_account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return await self._post(
            path_template(
                "/accounts/{path_account_id}/shares/{share_id}/recipients",
                path_account_id=path_account_id,
                share_id=share_id,
            ),
            body=await async_maybe_transform(
                {
                    "body_account_id": body_account_id,
                    "organization_id": organization_id,
                },
                recipient_create_params.RecipientCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RecipientCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RecipientCreateResponse]], ResultWrapper[RecipientCreateResponse]),
        )

    def list(
        self,
        share_id: str,
        *,
        account_id: str,
        include_resources: bool | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RecipientListResponse, AsyncV4PagePaginationArray[RecipientListResponse]]:
        """
        List share recipients by share ID.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          include_resources: Include resources in the response.

          page: Page number.

          per_page: Number of objects to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/recipients", account_id=account_id, share_id=share_id
            ),
            page=AsyncV4PagePaginationArray[RecipientListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_resources": include_resources,
                        "page": page,
                        "per_page": per_page,
                    },
                    recipient_list_params.RecipientListParams,
                ),
            ),
            model=RecipientListResponse,
        )

    async def delete(
        self,
        recipient_id: str,
        *,
        account_id: str,
        share_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RecipientDeleteResponse]:
        """
        Deletion is not immediate, an updated share recipient object with a new status
        will be returned.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          recipient_id: Share Recipient identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        if not recipient_id:
            raise ValueError(f"Expected a non-empty value for `recipient_id` but received {recipient_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/recipients/{recipient_id}",
                account_id=account_id,
                share_id=share_id,
                recipient_id=recipient_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RecipientDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RecipientDeleteResponse]], ResultWrapper[RecipientDeleteResponse]),
        )

    async def get(
        self,
        recipient_id: str,
        *,
        account_id: str,
        share_id: str,
        include_resources: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RecipientGetResponse]:
        """
        Get share recipient by ID.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          recipient_id: Share Recipient identifier tag.

          include_resources: Include resources in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        if not recipient_id:
            raise ValueError(f"Expected a non-empty value for `recipient_id` but received {recipient_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/recipients/{recipient_id}",
                account_id=account_id,
                share_id=share_id,
                recipient_id=recipient_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"include_resources": include_resources}, recipient_get_params.RecipientGetParams
                ),
                post_parser=ResultWrapper[Optional[RecipientGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RecipientGetResponse]], ResultWrapper[RecipientGetResponse]),
        )


class RecipientsResourceWithRawResponse:
    def __init__(self, recipients: RecipientsResource) -> None:
        self._recipients = recipients

        self.create = to_raw_response_wrapper(
            recipients.create,
        )
        self.list = to_raw_response_wrapper(
            recipients.list,
        )
        self.delete = to_raw_response_wrapper(
            recipients.delete,
        )
        self.get = to_raw_response_wrapper(
            recipients.get,
        )


class AsyncRecipientsResourceWithRawResponse:
    def __init__(self, recipients: AsyncRecipientsResource) -> None:
        self._recipients = recipients

        self.create = async_to_raw_response_wrapper(
            recipients.create,
        )
        self.list = async_to_raw_response_wrapper(
            recipients.list,
        )
        self.delete = async_to_raw_response_wrapper(
            recipients.delete,
        )
        self.get = async_to_raw_response_wrapper(
            recipients.get,
        )


class RecipientsResourceWithStreamingResponse:
    def __init__(self, recipients: RecipientsResource) -> None:
        self._recipients = recipients

        self.create = to_streamed_response_wrapper(
            recipients.create,
        )
        self.list = to_streamed_response_wrapper(
            recipients.list,
        )
        self.delete = to_streamed_response_wrapper(
            recipients.delete,
        )
        self.get = to_streamed_response_wrapper(
            recipients.get,
        )


class AsyncRecipientsResourceWithStreamingResponse:
    def __init__(self, recipients: AsyncRecipientsResource) -> None:
        self._recipients = recipients

        self.create = async_to_streamed_response_wrapper(
            recipients.create,
        )
        self.list = async_to_streamed_response_wrapper(
            recipients.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            recipients.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            recipients.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["InstantLogpushJob"]


class InstantLogpushJob(BaseModel):
    destination_conf: Optional[str] = None
    """Unique WebSocket address that will receive messages from Cloudflare’s edge."""

    fields: Optional[str] = None
    """Comma-separated list of fields."""

    filter: Optional[str] = None
    """Filters to drill down into specific events."""

    sample: Optional[int] = None
    """
    The sample parameter is the sample rate of the records set by the client:
    "sample": 1 is 100% of records "sample": 10 is 10% and so on.
    """

    session_id: Optional[str] = None
    """Unique session id of the job."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.zones.rate_plan_get_response import RatePlanGetResponse

__all__ = ["RatePlansResource", "AsyncRatePlansResource"]


class RatePlansResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RatePlansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RatePlansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RatePlansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RatePlansResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[RatePlanGetResponse]:
        """
        Lists all rate plans the zone can subscribe to.

        Args:
          zone_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/available_rate_plans", zone_id=zone_id),
            page=SyncSinglePage[RatePlanGetResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=RatePlanGetResponse,
        )


class AsyncRatePlansResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRatePlansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRatePlansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRatePlansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRatePlansResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RatePlanGetResponse, AsyncSinglePage[RatePlanGetResponse]]:
        """
        Lists all rate plans the zone can subscribe to.

        Args:
          zone_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/available_rate_plans", zone_id=zone_id),
            page=AsyncSinglePage[RatePlanGetResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=RatePlanGetResponse,
        )


class RatePlansResourceWithRawResponse:
    def __init__(self, rate_plans: RatePlansResource) -> None:
        self._rate_plans = rate_plans

        self.get = to_raw_response_wrapper(
            rate_plans.get,
        )


class AsyncRatePlansResourceWithRawResponse:
    def __init__(self, rate_plans: AsyncRatePlansResource) -> None:
        self._rate_plans = rate_plans

        self.get = async_to_raw_response_wrapper(
            rate_plans.get,
        )


class RatePlansResourceWithStreamingResponse:
    def __init__(self, rate_plans: RatePlansResource) -> None:
        self._rate_plans = rate_plans

        self.get = to_streamed_response_wrapper(
            rate_plans.get,
        )


class AsyncRatePlansResourceWithStreamingResponse:
    def __init__(self, rate_plans: AsyncRatePlansResource) -> None:
        self._rate_plans = rate_plans

        self.get = async_to_streamed_response_wrapper(
            rate_plans.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ....._types import Body, Query, Headers, NotGiven, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.zero_trust.tunnels.warp_connector import failover_update_params

__all__ = ["FailoverResource", "AsyncFailoverResource"]


class FailoverResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FailoverResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return FailoverResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FailoverResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return FailoverResourceWithStreamingResponse(self)

    def update(
        self,
        tunnel_id: str,
        *,
        account_id: str,
        client_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Triggers a manual failover for a specific WARP Connector Tunnel, setting the
        specified client as the active connector. The tunnel must be configured for high
        availability (HA) and the client must be linked to the tunnel.

        Args:
          account_id: Cloudflare account ID

          tunnel_id: UUID of the tunnel.

          client_id: UUID of the Cloudflare Tunnel connector.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tunnel_id:
            raise ValueError(f"Expected a non-empty value for `tunnel_id` but received {tunnel_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/warp_connector/{tunnel_id}/failover", account_id=account_id, tunnel_id=tunnel_id
            ),
            body=maybe_transform({"client_id": client_id}, failover_update_params.FailoverUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class AsyncFailoverResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFailoverResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFailoverResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFailoverResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncFailoverResourceWithStreamingResponse(self)

    async def update(
        self,
        tunnel_id: str,
        *,
        account_id: str,
        client_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Triggers a manual failover for a specific WARP Connector Tunnel, setting the
        specified client as the active connector. The tunnel must be configured for high
        availability (HA) and the client must be linked to the tunnel.

        Args:
          account_id: Cloudflare account ID

          tunnel_id: UUID of the tunnel.

          client_id: UUID of the Cloudflare Tunnel connector.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tunnel_id:
            raise ValueError(f"Expected a non-empty value for `tunnel_id` but received {tunnel_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/warp_connector/{tunnel_id}/failover", account_id=account_id, tunnel_id=tunnel_id
            ),
            body=await async_maybe_transform({"client_id": client_id}, failover_update_params.FailoverUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class FailoverResourceWithRawResponse:
    def __init__(self, failover: FailoverResource) -> None:
        self._failover = failover

        self.update = to_raw_response_wrapper(
            failover.update,
        )


class AsyncFailoverResourceWithRawResponse:
    def __init__(self, failover: AsyncFailoverResource) -> None:
        self._failover = failover

        self.update = async_to_raw_response_wrapper(
            failover.update,
        )


class FailoverResourceWithStreamingResponse:
    def __init__(self, failover: FailoverResource) -> None:
        self._failover = failover

        self.update = to_streamed_response_wrapper(
            failover.update,
        )


class AsyncFailoverResourceWithStreamingResponse:
    def __init__(self, failover: AsyncFailoverResource) -> None:
        self._failover = failover

        self.update = async_to_streamed_response_wrapper(
            failover.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tunnel import Tunnel as Tunnel
from .tunnel_param import TunnelParam as TunnelParam
from .keyless_certificate import KeylessCertificate as KeylessCertificate
from .keyless_certificate_edit_params import KeylessCertificateEditParams as KeylessCertificateEditParams
from .keyless_certificate_create_params import KeylessCertificateCreateParams as KeylessCertificateCreateParams
from .keyless_certificate_delete_response import KeylessCertificateDeleteResponse as KeylessCertificateDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .log_list_params import LogListParams as LogListParams
from .log_list_response import LogListResponse as LogListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel
from ..digital_experience_monitor import DigitalExperienceMonitor
from .http_tests.test_stat_over_time import TestStatOverTime

__all__ = [
    "Traceroute",
    "TracerouteStats",
    "TracerouteStatsAvailabilityPct",
    "TracerouteStatsAvailabilityPctSlot",
    "TracerouteStatsPacketLossPct",
    "TracerouteStatsPacketLossPctSlot",
    "TracerouteStatsByColo",
    "TracerouteStatsByColoAvailabilityPct",
    "TracerouteStatsByColoAvailabilityPctSlot",
    "TracerouteStatsByColoPacketLossPct",
    "TracerouteStatsByColoPacketLossPctSlot",
]


class TracerouteStatsAvailabilityPctSlot(BaseModel):
    timestamp: str

    value: float


class TracerouteStatsAvailabilityPct(BaseModel):
    slots: List[TracerouteStatsAvailabilityPctSlot]

    avg: Optional[float] = None
    """average observed in the time period"""

    max: Optional[float] = None
    """highest observed in the time period"""

    min: Optional[float] = None
    """lowest observed in the time period"""


class TracerouteStatsPacketLossPctSlot(BaseModel):
    timestamp: str

    value: float


class TracerouteStatsPacketLossPct(BaseModel):
    slots: List[TracerouteStatsPacketLossPctSlot]

    avg: Optional[float] = None
    """average observed in the time period"""

    max: Optional[float] = None
    """highest observed in the time period"""

    min: Optional[float] = None
    """lowest observed in the time period"""


class TracerouteStats(BaseModel):
    availability_pct: TracerouteStatsAvailabilityPct = FieldInfo(alias="availabilityPct")

    hops_count: TestStatOverTime = FieldInfo(alias="hopsCount")

    packet_loss_pct: TracerouteStatsPacketLossPct = FieldInfo(alias="packetLossPct")

    round_trip_time_ms: TestStatOverTime = FieldInfo(alias="roundTripTimeMs")

    unique_devices_total: int = FieldInfo(alias="uniqueDevicesTotal")
    """Count of unique devices that have run this test in the given time period"""


class TracerouteStatsByColoAvailabilityPctSlot(BaseModel):
    timestamp: str

    value: float


class TracerouteStatsByColoAvailabilityPct(BaseModel):
    slots: List[TracerouteStatsByColoAvailabilityPctSlot]

    avg: Optional[float] = None
    """average observed in the time period"""

    max: Optional[float] = None
    """highest observed in the time period"""

    min: Optional[float] = None
    """lowest observed in the time period"""


class TracerouteStatsByColoPacketLossPctSlot(BaseModel):
    timestamp: str

    value: float


class TracerouteStatsByColoPacketLossPct(BaseModel):
    slots: List[TracerouteStatsByColoPacketLossPctSlot]

    avg: Optional[float] = None
    """average observed in the time period"""

    max: Optional[float] = None
    """highest observed in the time period"""

    min: Optional[float] = None
    """lowest observed in the time period"""


class TracerouteStatsByColo(BaseModel):
    availability_pct: TracerouteStatsByColoAvailabilityPct = FieldInfo(alias="availabilityPct")

    colo: str

    hops_count: TestStatOverTime = FieldInfo(alias="hopsCount")

    packet_loss_pct: TracerouteStatsByColoPacketLossPct = FieldInfo(alias="packetLossPct")

    round_trip_time_ms: TestStatOverTime = FieldInfo(alias="roundTripTimeMs")

    unique_devices_total: int = FieldInfo(alias="uniqueDevicesTotal")
    """Count of unique devices that have run this test in the given time period"""


class Traceroute(BaseModel):
    host: str
    """The host of the Traceroute synthetic application test"""

    interval: str
    """The interval at which the Traceroute synthetic application test is set to run."""

    kind: Literal["traceroute"]

    name: str
    """The name of the Traceroute synthetic application test"""

    target_policies: Optional[List[DigitalExperienceMonitor]] = None

    targeted: Optional[bool] = None

    traceroute_stats: Optional[TracerouteStats] = FieldInfo(alias="tracerouteStats", default=None)

    traceroute_stats_by_colo: Optional[List[TracerouteStatsByColo]] = FieldInfo(
        alias="tracerouteStatsByColo", default=None
    )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["LeakedCredentialCheckCreateParams"]


class LeakedCredentialCheckCreateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Defines an identifier."""

    enabled: bool
    """Determines whether or not Leaked Credential Checks are enabled."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .origin_ca_certificates import (
    OriginCACertificatesResource,
    AsyncOriginCACertificatesResource,
    OriginCACertificatesResourceWithRawResponse,
    AsyncOriginCACertificatesResourceWithRawResponse,
    OriginCACertificatesResourceWithStreamingResponse,
    AsyncOriginCACertificatesResourceWithStreamingResponse,
)

__all__ = [
    "OriginCACertificatesResource",
    "AsyncOriginCACertificatesResource",
    "OriginCACertificatesResourceWithRawResponse",
    "AsyncOriginCACertificatesResourceWithRawResponse",
    "OriginCACertificatesResourceWithStreamingResponse",
    "AsyncOriginCACertificatesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["ConsumerScript"]


class ConsumerScript(BaseModel):
    """A reference to a script that will consume logs from the attached Worker."""

    service: str
    """Name of Worker that is to be the consumer."""

    environment: Optional[str] = None
    """Optional environment if the Worker utilizes one."""

    namespace: Optional[str] = None
    """Optional dispatch namespace the script belongs to."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import List, Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.radar.ai import (
    timeseries_group_summary_params,
    timeseries_group_timeseries_params,
    timeseries_group_user_agent_params,
    timeseries_group_timeseries_groups_params,
)
from ....types.radar.ai.timeseries_group_summary_response import TimeseriesGroupSummaryResponse
from ....types.radar.ai.timeseries_group_timeseries_response import TimeseriesGroupTimeseriesResponse
from ....types.radar.ai.timeseries_group_user_agent_response import TimeseriesGroupUserAgentResponse
from ....types.radar.ai.timeseries_group_timeseries_groups_response import TimeseriesGroupTimeseriesGroupsResponse

__all__ = ["TimeseriesGroupsResource", "AsyncTimeseriesGroupsResource"]


class TimeseriesGroupsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TimeseriesGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TimeseriesGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TimeseriesGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TimeseriesGroupsResourceWithStreamingResponse(self)

    @typing_extensions.deprecated(
        "Use [Radar > AI > Bots > Summary](https://developers.cloudflare.com/api/resources/radar/subresources/ai/subresources/bots/methods/summary_v2/) instead."
    )
    def summary(
        self,
        dimension: Literal[
            "USER_AGENT",
            "CRAWL_PURPOSE",
            "INDUSTRY",
            "VERTICAL",
            "CONTENT_TYPE",
            "RESPONSE_STATUS",
            "RESPONSE_STATUS_CATEGORY",
        ],
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        content_type: List[
            Literal[
                "HTML",
                "IMAGES",
                "JSON",
                "JAVASCRIPT",
                "CSS",
                "PLAIN_TEXT",
                "FONTS",
                "XML",
                "YAML",
                "VIDEO",
                "AUDIO",
                "MARKDOWN",
                "DOCUMENTS",
                "BINARY",
                "SERIALIZATION",
                "OTHER",
            ]
        ]
        | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        crawl_purpose: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        industry: SequenceNotStr[str] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        response_status: SequenceNotStr[str] | Omit = omit,
        response_status_category: List[
            Literal["INFORMATIONAL", "SUCCESS", "REDIRECTION", "CLIENT_ERROR", "SERVER_ERROR"]
        ]
        | Omit = omit,
        user_agent: SequenceNotStr[str] | Omit = omit,
        vertical: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TimeseriesGroupSummaryResponse:
        """
        Retrieves an aggregated summary of AI bots HTTP requests grouped by the
        specified dimension.

        Args:
          dimension: Specifies the attribute by which to group the results.

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          content_type: Filters results by content type category.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          crawl_purpose: Filters results by bot crawl purpose.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          industry: Filters results by industry.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          response_status: Filters results by HTTP response status code (e.g. 200, 403, 404). Only
              [IANA-registered codes](https://www.iana.org/assignments/http-status-codes/http-status-codes.xhtml)
              are accepted.

          response_status_category: Filters results by HTTP response status code category.

          user_agent: Filters results by user agent.

          vertical: Filters results by vertical.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return self._get(
            path_template("/radar/ai/bots/summary/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asn": asn,
                        "content_type": content_type,
                        "continent": continent,
                        "crawl_purpose": crawl_purpose,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "industry": industry,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                        "response_status": response_status,
                        "response_status_category": response_status_category,
                        "user_agent": user_agent,
                        "vertical": vertical,
                    },
                    timeseries_group_summary_params.TimeseriesGroupSummaryParams,
                ),
                post_parser=ResultWrapper[TimeseriesGroupSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[TimeseriesGroupSummaryResponse], ResultWrapper[TimeseriesGroupSummaryResponse]),
        )

    @typing_extensions.deprecated(
        "Use [Radar > AI > Bots > Timeseries](https://developers.cloudflare.com/api/resources/radar/subresources/ai/subresources/bots/methods/timeseries/) instead."
    )
    def timeseries(
        self,
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        asn: SequenceNotStr[str] | Omit = omit,
        content_type: List[
            Literal[
                "HTML",
                "IMAGES",
                "JSON",
                "JAVASCRIPT",
                "CSS",
                "PLAIN_TEXT",
                "FONTS",
                "XML",
                "YAML",
                "VIDEO",
                "AUDIO",
                "MARKDOWN",
                "DOCUMENTS",
                "BINARY",
                "SERIALIZATION",
                "OTHER",
            ]
        ]
        | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        crawl_purpose: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        industry: SequenceNotStr[str] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        response_status: SequenceNotStr[str] | Omit = omit,
        response_status_category: List[
            Literal["INFORMATIONAL", "SUCCESS", "REDIRECTION", "CLIENT_ERROR", "SERVER_ERROR"]
        ]
        | Omit = omit,
        user_agent: SequenceNotStr[str] | Omit = omit,
        vertical: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TimeseriesGroupTimeseriesResponse:
        """
        Retrieves AI bots HTTP request volume over time.

        Args:
          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          content_type: Filters results by content type category.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          crawl_purpose: Filters results by bot crawl purpose.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          industry: Filters results by industry.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          response_status: Filters results by HTTP response status code (e.g. 200, 403, 404). Only
              [IANA-registered codes](https://www.iana.org/assignments/http-status-codes/http-status-codes.xhtml)
              are accepted.

          response_status_category: Filters results by HTTP response status code category.

          user_agent: Filters results by user agent.

          vertical: Filters results by vertical.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/ai/bots/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "asn": asn,
                        "content_type": content_type,
                        "continent": continent,
                        "crawl_purpose": crawl_purpose,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "industry": industry,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                        "response_status": response_status,
                        "response_status_category": response_status_category,
                        "user_agent": user_agent,
                        "vertical": vertical,
                    },
                    timeseries_group_timeseries_params.TimeseriesGroupTimeseriesParams,
                ),
                post_parser=ResultWrapper[TimeseriesGroupTimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[TimeseriesGroupTimeseriesResponse], ResultWrapper[TimeseriesGroupTimeseriesResponse]),
        )

    @typing_extensions.deprecated(
        "Use [Radar > AI > Bots > Timeseries Groups](https://developers.cloudflare.com/api/resources/radar/subresources/ai/subresources/bots/methods/timeseries_groups/) instead."
    )
    def timeseries_groups(
        self,
        dimension: Literal[
            "USER_AGENT",
            "CRAWL_PURPOSE",
            "INDUSTRY",
            "VERTICAL",
            "CONTENT_TYPE",
            "RESPONSE_STATUS",
            "RESPONSE_STATUS_CATEGORY",
        ],
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        asn: SequenceNotStr[str] | Omit = omit,
        content_type: List[
            Literal[
                "HTML",
                "IMAGES",
                "JSON",
                "JAVASCRIPT",
                "CSS",
                "PLAIN_TEXT",
                "FONTS",
                "XML",
                "YAML",
                "VIDEO",
                "AUDIO",
                "MARKDOWN",
                "DOCUMENTS",
                "BINARY",
                "SERIALIZATION",
                "OTHER",
            ]
        ]
        | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        crawl_purpose: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        industry: SequenceNotStr[str] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        normalization: Literal["PERCENTAGE", "MIN0_MAX"] | Omit = omit,
        response_status: SequenceNotStr[str] | Omit = omit,
        response_status_category: List[
            Literal["INFORMATIONAL", "SUCCESS", "REDIRECTION", "CLIENT_ERROR", "SERVER_ERROR"]
        ]
        | Omit = omit,
        user_agent: SequenceNotStr[str] | Omit = omit,
        vertical: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TimeseriesGroupTimeseriesGroupsResponse:
        """
        Retrieves the distribution of HTTP requests from AI bots, grouped by the
        specified dimension over time.

        Args:
          dimension: Specifies the attribute by which to group the results.

          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          content_type: Filters results by content type category.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          crawl_purpose: Filters results by bot crawl purpose.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          industry: Filters results by industry.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          normalization: Normalization method applied to the results. Refer to
              [Normalization methods](https://developers.cloudflare.com/radar/concepts/normalization/).

          response_status: Filters results by HTTP response status code (e.g. 200, 403, 404). Only
              [IANA-registered codes](https://www.iana.org/assignments/http-status-codes/http-status-codes.xhtml)
              are accepted.

          response_status_category: Filters results by HTTP response status code category.

          user_agent: Filters results by user agent.

          vertical: Filters results by vertical.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return self._get(
            path_template("/radar/ai/bots/timeseries_groups/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "asn": asn,
                        "content_type": content_type,
                        "continent": continent,
                        "crawl_purpose": crawl_purpose,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "industry": industry,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                        "normalization": normalization,
                        "response_status": response_status,
                        "response_status_category": response_status_category,
                        "user_agent": user_agent,
                        "vertical": vertical,
                    },
                    timeseries_group_timeseries_groups_params.TimeseriesGroupTimeseriesGroupsParams,
                ),
                post_parser=ResultWrapper[TimeseriesGroupTimeseriesGroupsResponse]._unwrapper,
            ),
            cast_to=cast(
                Type[TimeseriesGroupTimeseriesGroupsResponse], ResultWrapper[TimeseriesGroupTimeseriesGroupsResponse]
            ),
        )

    @typing_extensions.deprecated(
        "Use [Radar AI Bots Timeseries Groups By Dimension](https://developers.cloudflare.com/api/resources/radar/subresources/ai/subresources/bots/methods/timeseries_groups/) instead."
    )
    def user_agent(
        self,
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TimeseriesGroupUserAgentResponse:
        """
        Retrieves the distribution of traffic by AI user agent over time.

        Args:
          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/ai/bots/timeseries_groups/user_agent",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                    },
                    timeseries_group_user_agent_params.TimeseriesGroupUserAgentParams,
                ),
                post_parser=ResultWrapper[TimeseriesGroupUserAgentResponse]._unwrapper,
            ),
            cast_to=cast(Type[TimeseriesGroupUserAgentResponse], ResultWrapper[TimeseriesGroupUserAgentResponse]),
        )


class AsyncTimeseriesGroupsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTimeseriesGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTimeseriesGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTimeseriesGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTimeseriesGroupsResourceWithStreamingResponse(self)

    @typing_extensions.deprecated(
        "Use [Radar > AI > Bots > Summary](https://developers.cloudflare.com/api/resources/radar/subresources/ai/subresources/bots/methods/summary_v2/) instead."
    )
    async def summary(
        self,
        dimension: Literal[
            "USER_AGENT",
            "CRAWL_PURPOSE",
            "INDUSTRY",
            "VERTICAL",
            "CONTENT_TYPE",
            "RESPONSE_STATUS",
            "RESPONSE_STATUS_CATEGORY",
        ],
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        content_type: List[
            Literal[
                "HTML",
                "IMAGES",
                "JSON",
                "JAVASCRIPT",
                "CSS",
                "PLAIN_TEXT",
                "FONTS",
                "XML",
                "YAML",
                "VIDEO",
                "AUDIO",
                "MARKDOWN",
                "DOCUMENTS",
                "BINARY",
                "SERIALIZATION",
                "OTHER",
            ]
        ]
        | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        crawl_purpose: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        industry: SequenceNotStr[str] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        response_status: SequenceNotStr[str] | Omit = omit,
        response_status_category: List[
            Literal["INFORMATIONAL", "SUCCESS", "REDIRECTION", "CLIENT_ERROR", "SERVER_ERROR"]
        ]
        | Omit = omit,
        user_agent: SequenceNotStr[str] | Omit = omit,
        vertical: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TimeseriesGroupSummaryResponse:
        """
        Retrieves an aggregated summary of AI bots HTTP requests grouped by the
        specified dimension.

        Args:
          dimension: Specifies the attribute by which to group the results.

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          content_type: Filters results by content type category.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          crawl_purpose: Filters results by bot crawl purpose.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          industry: Filters results by industry.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          response_status: Filters results by HTTP response status code (e.g. 200, 403, 404). Only
              [IANA-registered codes](https://www.iana.org/assignments/http-status-codes/http-status-codes.xhtml)
              are accepted.

          response_status_category: Filters results by HTTP response status code category.

          user_agent: Filters results by user agent.

          vertical: Filters results by vertical.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return await self._get(
            path_template("/radar/ai/bots/summary/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asn": asn,
                        "content_type": content_type,
                        "continent": continent,
                        "crawl_purpose": crawl_purpose,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "industry": industry,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                        "response_status": response_status,
                        "response_status_category": response_status_category,
                        "user_agent": user_agent,
                        "vertical": vertical,
                    },
                    timeseries_group_summary_params.TimeseriesGroupSummaryParams,
                ),
                post_parser=ResultWrapper[TimeseriesGroupSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[TimeseriesGroupSummaryResponse], ResultWrapper[TimeseriesGroupSummaryResponse]),
        )

    @typing_extensions.deprecated(
        "Use [Radar > AI > Bots > Timeseries](https://developers.cloudflare.com/api/resources/radar/subresources/ai/subresources/bots/methods/timeseries/) instead."
    )
    async def timeseries(
        self,
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        asn: SequenceNotStr[str] | Omit = omit,
        content_type: List[
            Literal[
                "HTML",
                "IMAGES",
                "JSON",
                "JAVASCRIPT",
                "CSS",
                "PLAIN_TEXT",
                "FONTS",
                "XML",
                "YAML",
                "VIDEO",
                "AUDIO",
                "MARKDOWN",
                "DOCUMENTS",
                "BINARY",
                "SERIALIZATION",
                "OTHER",
            ]
        ]
        | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        crawl_purpose: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        industry: SequenceNotStr[str] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        response_status: SequenceNotStr[str] | Omit = omit,
        response_status_category: List[
            Literal["INFORMATIONAL", "SUCCESS", "REDIRECTION", "CLIENT_ERROR", "SERVER_ERROR"]
        ]
        | Omit = omit,
        user_agent: SequenceNotStr[str] | Omit = omit,
        vertical: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TimeseriesGroupTimeseriesResponse:
        """
        Retrieves AI bots HTTP request volume over time.

        Args:
          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          content_type: Filters results by content type category.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          crawl_purpose: Filters results by bot crawl purpose.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          industry: Filters results by industry.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          response_status: Filters results by HTTP response status code (e.g. 200, 403, 404). Only
              [IANA-registered codes](https://www.iana.org/assignments/http-status-codes/http-status-codes.xhtml)
              are accepted.

          response_status_category: Filters results by HTTP response status code category.

          user_agent: Filters results by user agent.

          vertical: Filters results by vertical.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/ai/bots/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "asn": asn,
                        "content_type": content_type,
                        "continent": continent,
                        "crawl_purpose": crawl_purpose,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "industry": industry,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                        "response_status": response_status,
                        "response_status_category": response_status_category,
                        "user_agent": user_agent,
                        "vertical": vertical,
                    },
                    timeseries_group_timeseries_params.TimeseriesGroupTimeseriesParams,
                ),
                post_parser=ResultWrapper[TimeseriesGroupTimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[TimeseriesGroupTimeseriesResponse], ResultWrapper[TimeseriesGroupTimeseriesResponse]),
        )

    @typing_extensions.deprecated(
        "Use [Radar > AI > Bots > Timeseries Groups](https://developers.cloudflare.com/api/resources/radar/subresources/ai/subresources/bots/methods/timeseries_groups/) instead."
    )
    async def timeseries_groups(
        self,
        dimension: Literal[
            "USER_AGENT",
            "CRAWL_PURPOSE",
            "INDUSTRY",
            "VERTICAL",
            "CONTENT_TYPE",
            "RESPONSE_STATUS",
            "RESPONSE_STATUS_CATEGORY",
        ],
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        asn: SequenceNotStr[str] | Omit = omit,
        content_type: List[
            Literal[
                "HTML",
                "IMAGES",
                "JSON",
                "JAVASCRIPT",
                "CSS",
                "PLAIN_TEXT",
                "FONTS",
                "XML",
                "YAML",
                "VIDEO",
                "AUDIO",
                "MARKDOWN",
                "DOCUMENTS",
                "BINARY",
                "SERIALIZATION",
                "OTHER",
            ]
        ]
        | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        crawl_purpose: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        industry: SequenceNotStr[str] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        normalization: Literal["PERCENTAGE", "MIN0_MAX"] | Omit = omit,
        response_status: SequenceNotStr[str] | Omit = omit,
        response_status_category: List[
            Literal["INFORMATIONAL", "SUCCESS", "REDIRECTION", "CLIENT_ERROR", "SERVER_ERROR"]
        ]
        | Omit = omit,
        user_agent: SequenceNotStr[str] | Omit = omit,
        vertical: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TimeseriesGroupTimeseriesGroupsResponse:
        """
        Retrieves the distribution of HTTP requests from AI bots, grouped by the
        specified dimension over time.

        Args:
          dimension: Specifies the attribute by which to group the results.

          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          content_type: Filters results by content type category.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          crawl_purpose: Filters results by bot crawl purpose.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          industry: Filters results by industry.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          normalization: Normalization method applied to the results. Refer to
              [Normalization methods](https://developers.cloudflare.com/radar/concepts/normalization/).

          response_status: Filters results by HTTP response status code (e.g. 200, 403, 404). Only
              [IANA-registered codes](https://www.iana.org/assignments/http-status-codes/http-status-codes.xhtml)
              are accepted.

          response_status_category: Filters results by HTTP response status code category.

          user_agent: Filters results by user agent.

          vertical: Filters results by vertical.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return await self._get(
            path_template("/radar/ai/bots/timeseries_groups/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "asn": asn,
                        "content_type": content_type,
                        "continent": continent,
                        "crawl_purpose": crawl_purpose,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "industry": industry,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                        "normalization": normalization,
                        "response_status": response_status,
                        "response_status_category": response_status_category,
                        "user_agent": user_agent,
                        "vertical": vertical,
                    },
                    timeseries_group_timeseries_groups_params.TimeseriesGroupTimeseriesGroupsParams,
                ),
                post_parser=ResultWrapper[TimeseriesGroupTimeseriesGroupsResponse]._unwrapper,
            ),
            cast_to=cast(
                Type[TimeseriesGroupTimeseriesGroupsResponse], ResultWrapper[TimeseriesGroupTimeseriesGroupsResponse]
            ),
        )

    @typing_extensions.deprecated(
        "Use [Radar AI Bots Timeseries Groups By Dimension](https://developers.cloudflare.com/api/resources/radar/subresources/ai/subresources/bots/methods/timeseries_groups/) instead."
    )
    async def user_agent(
        self,
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TimeseriesGroupUserAgentResponse:
        """
        Retrieves the distribution of traffic by AI user agent over time.

        Args:
          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/ai/bots/timeseries_groups/user_agent",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                    },
                    timeseries_group_user_agent_params.TimeseriesGroupUserAgentParams,
                ),
                post_parser=ResultWrapper[TimeseriesGroupUserAgentResponse]._unwrapper,
            ),
            cast_to=cast(Type[TimeseriesGroupUserAgentResponse], ResultWrapper[TimeseriesGroupUserAgentResponse]),
        )


class TimeseriesGroupsResourceWithRawResponse:
    def __init__(self, timeseries_groups: TimeseriesGroupsResource) -> None:
        self._timeseries_groups = timeseries_groups

        self.summary = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                timeseries_groups.summary,  # pyright: ignore[reportDeprecated],
            )
        )
        self.timeseries = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                timeseries_groups.timeseries,  # pyright: ignore[reportDeprecated],
            )
        )
        self.timeseries_groups = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                timeseries_groups.timeseries_groups,  # pyright: ignore[reportDeprecated],
            )
        )
        self.user_agent = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                timeseries_groups.user_agent,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncTimeseriesGroupsResourceWithRawResponse:
    def __init__(self, timeseries_groups: AsyncTimeseriesGroupsResource) -> None:
        self._timeseries_groups = timeseries_groups

        self.summary = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                timeseries_groups.summary,  # pyright: ignore[reportDeprecated],
            )
        )
        self.timeseries = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                timeseries_groups.timeseries,  # pyright: ignore[reportDeprecated],
            )
        )
        self.timeseries_groups = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                timeseries_groups.timeseries_groups,  # pyright: ignore[reportDeprecated],
            )
        )
        self.user_agent = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                timeseries_groups.user_agent,  # pyright: ignore[reportDeprecated],
            )
        )


class TimeseriesGroupsResourceWithStreamingResponse:
    def __init__(self, timeseries_groups: TimeseriesGroupsResource) -> None:
        self._timeseries_groups = timeseries_groups

        self.summary = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                timeseries_groups.summary,  # pyright: ignore[reportDeprecated],
            )
        )
        self.timeseries = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                timeseries_groups.timeseries,  # pyright: ignore[reportDeprecated],
            )
        )
        self.timeseries_groups = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                timeseries_groups.timeseries_groups,  # pyright: ignore[reportDeprecated],
            )
        )
        self.user_agent = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                timeseries_groups.user_agent,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncTimeseriesGroupsResourceWithStreamingResponse:
    def __init__(self, timeseries_groups: AsyncTimeseriesGroupsResource) -> None:
        self._timeseries_groups = timeseries_groups

        self.summary = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                timeseries_groups.summary,  # pyright: ignore[reportDeprecated],
            )
        )
        self.timeseries = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                timeseries_groups.timeseries,  # pyright: ignore[reportDeprecated],
            )
        )
        self.timeseries_groups = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                timeseries_groups.timeseries_groups,  # pyright: ignore[reportDeprecated],
            )
        )
        self.user_agent = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                timeseries_groups.user_agent,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SilenceCreateResponse", "Error", "Message"]


class Error(BaseModel):
    message: str

    code: Optional[int] = None


class Message(BaseModel):
    message: str

    code: Optional[int] = None


class SilenceCreateResponse(BaseModel):
    errors: List[Error]

    messages: List[Message]

    success: Literal[True]
    """Whether the API call was successful"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["AccessRuleListParams", "Configuration"]


class AccessRuleListParams(TypedDict, total=False):
    account_id: str
    """The Account ID to use for this endpoint. Mutually exclusive with the Zone ID."""

    zone_id: str
    """The Zone ID to use for this endpoint. Mutually exclusive with the Account ID."""

    configuration: Configuration

    direction: Literal["asc", "desc"]
    """Defines the direction used to sort returned rules."""

    match: Literal["any", "all"]
    """Defines the search requirements.

    When set to `all`, all the search requirements must match. When set to `any`,
    only one of the search requirements has to match.
    """

    mode: Literal["block", "challenge", "whitelist", "js_challenge", "managed_challenge"]
    """The action to apply to a matched request."""

    notes: str
    """
    Defines the string to search for in the notes of existing IP Access rules.
    Notes: For example, the string 'attack' would match IP Access rules with notes
    'Attack 26/02' and 'Attack 27/02'. The search is case insensitive.
    """

    order: Literal["configuration.target", "configuration.value", "mode"]
    """Defines the field used to sort returned rules."""

    page: float
    """Defines the requested page within paginated list of results."""

    per_page: float
    """Defines the maximum number of results requested."""


class Configuration(TypedDict, total=False):
    target: Literal["ip", "ip_range", "asn", "country"]
    """Defines the target to search in existing rules."""

    value: str
    """
    Defines the target value to search for in existing rules: an IP address, an IP
    address range, or a country code, depending on the provided
    `configuration.target`. Notes: You can search for a single IPv4 address, an IP
    address range with a subnet of '/16' or '/24', or a two-letter ISO-3166-1
    alpha-2 country code.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.zero_trust.devices import setting_edit_params, setting_update_params
from ....types.zero_trust.devices.device_settings import DeviceSettings

__all__ = ["SettingsResource", "AsyncSettingsResource"]


class SettingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SettingsResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        account_id: str,
        disable_for_time: float | Omit = omit,
        external_emergency_signal_enabled: bool | Omit = omit,
        external_emergency_signal_fingerprint: str | Omit = omit,
        external_emergency_signal_interval: str | Omit = omit,
        external_emergency_signal_url: str | Omit = omit,
        gateway_proxy_enabled: bool | Omit = omit,
        gateway_udp_proxy_enabled: bool | Omit = omit,
        root_certificate_installation_enabled: bool | Omit = omit,
        use_zt_virtual_ip: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DeviceSettings]:
        """
        Updates the current device settings for a Zero Trust account.

        Args:
          disable_for_time: Sets the time limit, in seconds, that a user can use an override code to bypass
              WARP.

          external_emergency_signal_enabled: Controls whether the external emergency disconnect feature is enabled.

          external_emergency_signal_fingerprint: The SHA256 fingerprint (64 hexadecimal characters) of the HTTPS server
              certificate for the external_emergency_signal_url. If provided, the WARP client
              will use this value to verify the server's identity. The device will ignore any
              response if the server's certificate fingerprint does not exactly match this
              value.

          external_emergency_signal_interval: The interval at which the WARP client fetches the emergency disconnect signal,
              formatted as a duration string (e.g., "5m", "2m30s", "1h"). Minimum 30 seconds.

          external_emergency_signal_url: The HTTPS URL from which to fetch the emergency disconnect signal. Must use
              HTTPS and have an IPv4 or IPv6 address as the host.

          gateway_proxy_enabled: Enable gateway proxy filtering on TCP.

          gateway_udp_proxy_enabled: Enable gateway proxy filtering on UDP.

          root_certificate_installation_enabled: Enable installation of cloudflare managed root certificate.

          use_zt_virtual_ip: Enable using CGNAT virtual IPv4.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/devices/settings", account_id=account_id),
            body=maybe_transform(
                {
                    "disable_for_time": disable_for_time,
                    "external_emergency_signal_enabled": external_emergency_signal_enabled,
                    "external_emergency_signal_fingerprint": external_emergency_signal_fingerprint,
                    "external_emergency_signal_interval": external_emergency_signal_interval,
                    "external_emergency_signal_url": external_emergency_signal_url,
                    "gateway_proxy_enabled": gateway_proxy_enabled,
                    "gateway_udp_proxy_enabled": gateway_udp_proxy_enabled,
                    "root_certificate_installation_enabled": root_certificate_installation_enabled,
                    "use_zt_virtual_ip": use_zt_virtual_ip,
                },
                setting_update_params.SettingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DeviceSettings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DeviceSettings]], ResultWrapper[DeviceSettings]),
        )

    def delete(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DeviceSettings]:
        """
        Resets the current device settings for a Zero Trust account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/devices/settings", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DeviceSettings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DeviceSettings]], ResultWrapper[DeviceSettings]),
        )

    def edit(
        self,
        *,
        account_id: str,
        disable_for_time: float | Omit = omit,
        external_emergency_signal_enabled: bool | Omit = omit,
        external_emergency_signal_fingerprint: str | Omit = omit,
        external_emergency_signal_interval: str | Omit = omit,
        external_emergency_signal_url: str | Omit = omit,
        gateway_proxy_enabled: bool | Omit = omit,
        gateway_udp_proxy_enabled: bool | Omit = omit,
        root_certificate_installation_enabled: bool | Omit = omit,
        use_zt_virtual_ip: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DeviceSettings]:
        """
        Patches the current device settings for a Zero Trust account.

        Args:
          disable_for_time: Sets the time limit, in seconds, that a user can use an override code to bypass
              WARP.

          external_emergency_signal_enabled: Controls whether the external emergency disconnect feature is enabled.

          external_emergency_signal_fingerprint: The SHA256 fingerprint (64 hexadecimal characters) of the HTTPS server
              certificate for the external_emergency_signal_url. If provided, the WARP client
              will use this value to verify the server's identity. The device will ignore any
              response if the server's certificate fingerprint does not exactly match this
              value.

          external_emergency_signal_interval: The interval at which the WARP client fetches the emergency disconnect signal,
              formatted as a duration string (e.g., "5m", "2m30s", "1h"). Minimum 30 seconds.

          external_emergency_signal_url: The HTTPS URL from which to fetch the emergency disconnect signal. Must use
              HTTPS and have an IPv4 or IPv6 address as the host.

          gateway_proxy_enabled: Enable gateway proxy filtering on TCP.

          gateway_udp_proxy_enabled: Enable gateway proxy filtering on UDP.

          root_certificate_installation_enabled: Enable installation of cloudflare managed root certificate.

          use_zt_virtual_ip: Enable using CGNAT virtual IPv4.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._patch(
            path_template("/accounts/{account_id}/devices/settings", account_id=account_id),
            body=maybe_transform(
                {
                    "disable_for_time": disable_for_time,
                    "external_emergency_signal_enabled": external_emergency_signal_enabled,
                    "external_emergency_signal_fingerprint": external_emergency_signal_fingerprint,
                    "external_emergency_signal_interval": external_emergency_signal_interval,
                    "external_emergency_signal_url": external_emergency_signal_url,
                    "gateway_proxy_enabled": gateway_proxy_enabled,
                    "gateway_udp_proxy_enabled": gateway_udp_proxy_enabled,
                    "root_certificate_installation_enabled": root_certificate_installation_enabled,
                    "use_zt_virtual_ip": use_zt_virtual_ip,
                },
                setting_edit_params.SettingEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DeviceSettings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DeviceSettings]], ResultWrapper[DeviceSettings]),
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DeviceSettings]:
        """
        Describes the current device settings for a Zero Trust account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/devices/settings", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DeviceSettings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DeviceSettings]], ResultWrapper[DeviceSettings]),
        )


class AsyncSettingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSettingsResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        account_id: str,
        disable_for_time: float | Omit = omit,
        external_emergency_signal_enabled: bool | Omit = omit,
        external_emergency_signal_fingerprint: str | Omit = omit,
        external_emergency_signal_interval: str | Omit = omit,
        external_emergency_signal_url: str | Omit = omit,
        gateway_proxy_enabled: bool | Omit = omit,
        gateway_udp_proxy_enabled: bool | Omit = omit,
        root_certificate_installation_enabled: bool | Omit = omit,
        use_zt_virtual_ip: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DeviceSettings]:
        """
        Updates the current device settings for a Zero Trust account.

        Args:
          disable_for_time: Sets the time limit, in seconds, that a user can use an override code to bypass
              WARP.

          external_emergency_signal_enabled: Controls whether the external emergency disconnect feature is enabled.

          external_emergency_signal_fingerprint: The SHA256 fingerprint (64 hexadecimal characters) of the HTTPS server
              certificate for the external_emergency_signal_url. If provided, the WARP client
              will use this value to verify the server's identity. The device will ignore any
              response if the server's certificate fingerprint does not exactly match this
              value.

          external_emergency_signal_interval: The interval at which the WARP client fetches the emergency disconnect signal,
              formatted as a duration string (e.g., "5m", "2m30s", "1h"). Minimum 30 seconds.

          external_emergency_signal_url: The HTTPS URL from which to fetch the emergency disconnect signal. Must use
              HTTPS and have an IPv4 or IPv6 address as the host.

          gateway_proxy_enabled: Enable gateway proxy filtering on TCP.

          gateway_udp_proxy_enabled: Enable gateway proxy filtering on UDP.

          root_certificate_installation_enabled: Enable installation of cloudflare managed root certificate.

          use_zt_virtual_ip: Enable using CGNAT virtual IPv4.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/devices/settings", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "disable_for_time": disable_for_time,
                    "external_emergency_signal_enabled": external_emergency_signal_enabled,
                    "external_emergency_signal_fingerprint": external_emergency_signal_fingerprint,
                    "external_emergency_signal_interval": external_emergency_signal_interval,
                    "external_emergency_signal_url": external_emergency_signal_url,
                    "gateway_proxy_enabled": gateway_proxy_enabled,
                    "gateway_udp_proxy_enabled": gateway_udp_proxy_enabled,
                    "root_certificate_installation_enabled": root_certificate_installation_enabled,
                    "use_zt_virtual_ip": use_zt_virtual_ip,
                },
                setting_update_params.SettingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DeviceSettings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DeviceSettings]], ResultWrapper[DeviceSettings]),
        )

    async def delete(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DeviceSettings]:
        """
        Resets the current device settings for a Zero Trust account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/devices/settings", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DeviceSettings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DeviceSettings]], ResultWrapper[DeviceSettings]),
        )

    async def edit(
        self,
        *,
        account_id: str,
        disable_for_time: float | Omit = omit,
        external_emergency_signal_enabled: bool | Omit = omit,
        external_emergency_signal_fingerprint: str | Omit = omit,
        external_emergency_signal_interval: str | Omit = omit,
        external_emergency_signal_url: str | Omit = omit,
        gateway_proxy_enabled: bool | Omit = omit,
        gateway_udp_proxy_enabled: bool | Omit = omit,
        root_certificate_installation_enabled: bool | Omit = omit,
        use_zt_virtual_ip: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DeviceSettings]:
        """
        Patches the current device settings for a Zero Trust account.

        Args:
          disable_for_time: Sets the time limit, in seconds, that a user can use an override code to bypass
              WARP.

          external_emergency_signal_enabled: Controls whether the external emergency disconnect feature is enabled.

          external_emergency_signal_fingerprint: The SHA256 fingerprint (64 hexadecimal characters) of the HTTPS server
              certificate for the external_emergency_signal_url. If provided, the WARP client
              will use this value to verify the server's identity. The device will ignore any
              response if the server's certificate fingerprint does not exactly match this
              value.

          external_emergency_signal_interval: The interval at which the WARP client fetches the emergency disconnect signal,
              formatted as a duration string (e.g., "5m", "2m30s", "1h"). Minimum 30 seconds.

          external_emergency_signal_url: The HTTPS URL from which to fetch the emergency disconnect signal. Must use
              HTTPS and have an IPv4 or IPv6 address as the host.

          gateway_proxy_enabled: Enable gateway proxy filtering on TCP.

          gateway_udp_proxy_enabled: Enable gateway proxy filtering on UDP.

          root_certificate_installation_enabled: Enable installation of cloudflare managed root certificate.

          use_zt_virtual_ip: Enable using CGNAT virtual IPv4.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._patch(
            path_template("/accounts/{account_id}/devices/settings", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "disable_for_time": disable_for_time,
                    "external_emergency_signal_enabled": external_emergency_signal_enabled,
                    "external_emergency_signal_fingerprint": external_emergency_signal_fingerprint,
                    "external_emergency_signal_interval": external_emergency_signal_interval,
                    "external_emergency_signal_url": external_emergency_signal_url,
                    "gateway_proxy_enabled": gateway_proxy_enabled,
                    "gateway_udp_proxy_enabled": gateway_udp_proxy_enabled,
                    "root_certificate_installation_enabled": root_certificate_installation_enabled,
                    "use_zt_virtual_ip": use_zt_virtual_ip,
                },
                setting_edit_params.SettingEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DeviceSettings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DeviceSettings]], ResultWrapper[DeviceSettings]),
        )

    async def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DeviceSettings]:
        """
        Describes the current device settings for a Zero Trust account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/devices/settings", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DeviceSettings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DeviceSettings]], ResultWrapper[DeviceSettings]),
        )


class SettingsResourceWithRawResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.update = to_raw_response_wrapper(
            settings.update,
        )
        self.delete = to_raw_response_wrapper(
            settings.delete,
        )
        self.edit = to_raw_response_wrapper(
            settings.edit,
        )
        self.get = to_raw_response_wrapper(
            settings.get,
        )


class AsyncSettingsResourceWithRawResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.update = async_to_raw_response_wrapper(
            settings.update,
        )
        self.delete = async_to_raw_response_wrapper(
            settings.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            settings.edit,
        )
        self.get = async_to_raw_response_wrapper(
            settings.get,
        )


class SettingsResourceWithStreamingResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.update = to_streamed_response_wrapper(
            settings.update,
        )
        self.delete = to_streamed_response_wrapper(
            settings.delete,
        )
        self.edit = to_streamed_response_wrapper(
            settings.edit,
        )
        self.get = to_streamed_response_wrapper(
            settings.get,
        )


class AsyncSettingsResourceWithStreamingResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.update = async_to_streamed_response_wrapper(
            settings.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            settings.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            settings.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            settings.get,
        )
